@@ -125,7 +125,7 @@ sub blob_params {
     my $type = shift;
 
     # Don't assign to key 'value' as it is defined later.
-    return ( { pg_type => DBD::Pg::PG_BYTEA() } ) if $type eq "blob";
+    return ( { pg_type => DBD::Pg::PG_BYTEA() } ) if $type =~ /^(?:blob|bytea)$/;
     return ( {} );
 }
 
@@ -172,9 +172,13 @@ sub _make_clause_case_insensitive {
     my $value    = shift;
 
     if ($self->_case_insensitivity_valid($column, $operator, $value)) {
-        if ( $operator =~ /(?:LIKE|=)/i ) {
+        if ( $operator =~ /(?:LIKE|=|IN)/i ) {
             $column = "LOWER($column)";
-            $value = "LOWER($value)";
+            if ($operator eq 'IN' and ref($value) eq 'ARRAY') {
+                $value = [map { "LOWER($_)" } @$value];
+            } else {
+                $value = "LOWER($value)";
+            }
         }
     }
     return ( $column, $operator, $value );
@@ -206,9 +210,13 @@ sub distinct_query {
       = [ @{ $sb->{group_by} || [] }, { column => 'id' } ];
     local $sb->{order_by} = [
       map {
-            ( $_->{alias} and $_->{alias} ne "main" )
-          ? { %{$_}, column => "min(" . $_->{column} . ")" }
-          : $_
+          my $alias = $_->{alias} || '';
+          my $column = $_->{column};
+          $alias .= '.' if $alias;
+          #warn "alias $alias => column $column\n";
+            ((!$alias or $alias eq 'main.') and $column eq 'id')
+          ? $_
+          : { %{$_}, alias => '', column => "min($alias$column)" }
         } @{ $sb->{order_by} }
     ];
     my $group = $sb->_group_clause;
