@@ -77,13 +77,13 @@ sub case_sensitive {
     return (1);
 }
 
-sub apply_limits {
-    my $self         = shift;
-    my $statementref = shift;
-    my $per_page     = shift;
-    my $first        = shift;
-
-}
+# sub apply_limits {
+#     my $self         = shift;
+#     my $statementref = shift;
+#     my $per_page     = shift;
+#     my $first        = shift;
+# 
+# }
 
 =head2 distinct_query STATEMENTREF
 
@@ -115,18 +115,6 @@ sub distinct_query {
     $$statementref .= $sb->_order_clause;
 }
 
-=head2 binary_safe_blobs
-
-Return undef, as Oracle doesn't support binary-safe CLOBS
-
-
-=cut
-
-sub binary_safe_blobs {
-    my $self = shift;
-    return (undef);
-}
-
 1;
 
 __END__
